#!/usr/bin/env python

# Copyright 2018 GARR CSD.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import string

from charms.leadership import leader_get, leader_set

from subprocess import check_call, check_output

from charms import layer
from charms.reactive import hook
from charms.reactive import remove_state
from charms.reactive import set_state
from charms.reactive import is_state
from charms.reactive import when, when_any, when_not, when_none
from charms.reactive.helpers import data_changed, any_file_changed

from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core.host import service_stop
from charmhelpers.core.templating import render
from charmhelpers.fetch import archiveurl


# Webhook configuration files

webhook_authn_config_path = '/root/cdk/webhook-authn.config'
webhook_authz_config_path = '/root/cdk/webhook-authz.config'
webhook_sync_config_path = '/root/cdk/sync-config.yaml'
webhook_policy_file = '/root/cdk/webhook-policy.json'

k8s_keystone_auth_start = '/root/cdk/k8s-keystone-auth.sh'
k8s_keystone_auth_path = '/root/cdk/k8s-keystone-auth'
k8s_keystone_auth_service = '/etc/systemd/system/k8s-keystone-auth.service'

@when_not('authentication.setup')
def setup_authentication():
    '''Setup Webhook authentication access for the cluster.'''

    hookenv.status_set('maintenance', 'Setting up Webhook authentication.')
    
    # Render the webhook configuration from the webhook templates
    # using these template parameters:
    webhook_context = {
        'authn_server_url':  
        hookenv.config('authn-server-url'),
        'authz_server_url':  
        hookenv.config('authz-server-url'),
        'k8s_keystone_auth_path':
        k8s_keystone_auth_path,
        'k8s_keystone_auth_start':
        k8s_keystone_auth_start,
        'keystone_url':
        hookenv.config('keystone-url'),
        'keystone_policy': 
        webhook_policy_file
    }
    render('webhook-authn.config', webhook_authn_config_path, webhook_context)
    render('webhook-authz.config', webhook_authz_config_path, webhook_context)
    render('sync-config.yaml', webhook_sync_config_path, webhook_context)
    render('webhook-policy.json', webhook_policy_file, webhook_context)

    # the systemd unit for deploying the k8s_keystone_auth service.
    render('k8s-keystone-auth.service', k8s_keystone_auth_service, webhook_context)
    # invokes this script
    render('k8s-keystone-auth.sh', k8s_keystone_auth_start, webhook_context)
    # that starts this service
    handler = archiveurl.ArchiveUrlFetchHandler()
    k8s_keystone_auth_url = hookenv.config('k8s-keystone-auth-url')
    hookenv.log('Downloading %s' % k8s_keystone_auth_url)
    handler.download(k8s_keystone_auth_url, k8s_keystone_auth_path)

    check_call(['chmod', '+x', k8s_keystone_auth_path])
    check_call(['chmod', '+x', k8s_keystone_auth_start])

    # CHECKME: do we need authorization-mode Webhook?
    # get current value for authorization-mode
    cmd = ['snap', 'get', 'kube-apiserver', 'authorization-mode']
    auth_mode = check_output(cmd).decode('utf-8').strip()
    if 'Webhook' not in auth_mode:
        if auth_mode: auth_mode += ','
        auth_mode += 'Webhook'
    cmd = ['snap', 'set', 'kube-apiserver',
           'authorization-mode=%s' % auth_mode,
           'authorization-webhook-config-file=%s' % webhook_authz_config_path,
           'authentication-token-webhook-config-file=%s' % webhook_authn_config_path]
    hookenv.log('Configuring kube-apiserver: %s' % ' '.join(cmd))
    check_call(cmd)
                    
    set_state('authentication.setup')

@when('leadership.is_leader')
@when('authentication.setup')
def setup_authentication_server():
    ''' Start k8s_keystone_auth '''


    check_call(['chmod', '+x', k8s_keystone_auth_path])
    check_call(['chmod', '+x', k8s_keystone_auth_start])

    hookenv.status_set('maintenance', 'Starting authentication server.')

    command = ['systemctl', 'enable', 'k8s-keystone-auth']
    check_call(command)


    command = ['systemctl', 'restart', 'k8s-keystone-auth.service']
    check_call(command)


    set_state('authentication.server')


@hookenv.atexit
def set_final_status():
    ''' Set the final status of the charm as we leave hook execution '''
    try:
        goal_state = hookenv.goal_state()
    except NotImplementedError:
        goal_state = {}

    is_leader = is_state('leadership.is_leader')
    authentication_server = is_state('authentication.server')
    if not is_leader and not authentication_server:
        hookenv.status_set('waiting', 'Waiting for authentication server.')
        return

    hookenv.status_set('active', 'Webhook server running.')


